xquery version "3.1";
(:
 : Error catalogues.
 : Copyright© Mary Holstege 2020-2021
 : CC-BY (https://creativecommons.org/licenses/by/4.0/)
 :)
module namespace this="http://mathling.com/core/errors"; 

declare namespace map="http://www.w3.org/2005/xpath-functions/map";
declare namespace array="http://www.w3.org/2005/xpath-functions/array";
declare namespace math="http://www.w3.org/2005/xpath-functions/math";
declare namespace errors="http://mathling.com/errors";

declare variable $this:ERRORNS as xs:string := "http://mathling.com/errors";
declare variable $this:MESSAGES as xs:string := "../Messages/";
declare variable $this:LANG as xs:string := "en";
declare variable $this:QUOTE-LIMIT as xs:integer := 20;

declare function this:error(
  $code as xs:string
)
{
  this:error($code, ())
};

declare function this:error(
  $code as xs:string,
  $data as item()*
)
{
  let $catalog :=
    try {
      doc($this:MESSAGES||substring-before($code,"-")||"-"||$this:LANG||".xml")
    } catch * { () }
  let $error := $catalog/errors:errors/errors:error[errors:code=$code]
  return
    if (empty($error))
    then error(QName($this:ERRORNS,"ml:"||$code), "Error("||this:quote($data, $this:QUOTE-LIMIT)||")")
    else error(QName($this:ERRORNS,"ml:"||$code), this:format($error, $data))
};

declare function this:format(
  $error as element(errors:error),
  $data as item()*
) as xs:string
{
  fold-left(1 to count($data), string($error/errors:format),
    function($string as xs:string, $i as xs:integer) as xs:string {
      replace($string, "%"||$i, this:quote($data[$i], $this:QUOTE-LIMIT))
    }
  )
};

declare function this:quote($data as item()*) as xs:string
{
  string-join(
    for $val in $data return typeswitch($val)
    case xs:QName return (
      let $prefix := prefix-from-QName($val)
      let $uri := namespace-uri-from-QName($val)
      let $lc := local-name-from-QName($val)
      return
        if (empty($prefix)) then (
          if (empty($uri)) then $lc
          else "{"||$uri||"}"||$lc
        ) else (
          $prefix||":"||$lc
        )
    )
    case map(*) return (
      "{"||(
        string-join(
          map:for-each($val, function($k,$v) {$k||":"||this:quote($v)}),
          ","
        )
      )||"}"
    )
    case array(*) return (
      "["||(
        string-join(array:for-each($val, this:quote#1), ",")
      )||"]"
    )
    case function(*) return (
      "function "||this:quote(function-name($val))||"("||(
        string-join(
          for $i in 1 to function-arity($val) return "v"||$i,
          ","
        )
      )||")"
    )
    case document-node() return (
      "document {"||this:quote($val/node())||"}"
    )
    case node() return (
      serialize($val)
    )
    default return string($val)
    ,
    ","
  )
};

declare function this:quote($data as item()*, $limit as xs:integer) as xs:string
{
  let $str := this:quote($data)
  return (
    substring($str, 1, $limit)||(
      if (string-length($str) > $limit) then "..." else ""
    )
  )
};

(: errors.xqy :)
