xquery version "3.1";
(:
 : Space: rectangular extent with possible margin
 :
 : Copyright© Mary Holstege 2020-2021
 : CC-BY (https://creativecommons.org/licenses/by/4.0/)
 :)
module namespace this="http://mathling.com/type/space"; 

import module namespace util="http://mathling.com/core/utilities"
       at "../core/utilities.xqy";
import module namespace errors="http://mathling.com/core/errors"
       at "../core/errors.xqy";

declare namespace map="http://www.w3.org/2005/xpath-functions/map";
declare namespace array="http://www.w3.org/2005/xpath-functions/array";
declare namespace math="http://www.w3.org/2005/xpath-functions/math";

declare variable $this:RESERVED := ("kind", "height", "width", "edge");

(:======================================================================
 : Constructors
 :======================================================================:)

(:
 : space()
 : Construct a rectangular space with no margin
 :
 : $width: width
 : $height: height
 :)
declare function this:space($width as xs:double, $height as xs:double) as map(xs:string,item()*)
{
  map {
    "kind": "space",
    "width": $width,
    "height": $height,
    "edge": 0
  }
};

(:
 : space()
 : Construct a rectangular space with a margin
 :
 : $width: width
 : $height: height
 : $edge: margin on the edges
 :)
declare function this:space(
  $width as xs:double,
  $height as xs:double,
  $edge as xs:double
) as map(xs:string,item()*)
{
  map {
    "kind": "space",
    "width": $width,
    "height": $height,
    "edge": $edge
  }
};

(:======================================================================
 : Accessors
 :======================================================================:)

(: 
 : width()
 : Accessor for the width of the space
 :
 : $space: the space
 :)
declare function this:width($space as map(xs:string,item()*)) as xs:double
{
  $space("width")
};

(: 
 : height()
 : Accessor for the height of the space
 :
 : $space: the space
 :)
declare function this:height($space as map(xs:string,item()*)) as xs:double
{
  $space("height")
};

(: 
 : edge()
 : Accessor for the edge of the space
 :
 : $space: the space
 :)
declare function this:edge($space as map(xs:string,item()*)) as xs:double
{
  $space("edge")
};

(: 
 : kind()
 : Accessor for the kind of data this is; for debugging
 :
 : $space: the space
 :)
declare function this:kind($space as map(xs:string,item()*)) as xs:string
{
  $space("kind")
};

(: space :)
